function c=autocov(y,lags)
% AUTOCORR      Autocovariance function (normalized)
%
% c = autocorr(y,M,opt)
%
% Results are plotted if no output arguments are given.
%
%   Jenkins,G.M. and Watts,D.G., 1969. Spectral analysis and its
%   applications. Holden-Day series in time series analysis, 1, 513 pp.
%
% See also XCORR CORRCOEF COV ITS
if ~all([length(size(y))==2 , ismember(1, size(y))]);
    error('y must be a vector');
end

if nargin == 1;
    lags = 0:(length(y)-2);
end;

nlag = length(lags);
ntim = length(y);
c = repmat(NaN, [nlag 1]);

for i = 1:nlag;
    ind1 = 1:(ntim-abs(lags(i)));
    ind2 = (1+abs(lags(i))):ntim;
    tem = cov(y(ind1), y(ind2));
    c(i) = tem(1,2);
end

