clear all;

nrTest=3;   %The first 3 tests are simple functions
            %The last 5 tests are taken from "On the Numerical Inversion of LAlace Transform: Comparison of three NEw Methods on Characteristic Problems from Applications",
            %Dean Duffy, ACM TOMS, VOl. 19 pp.35-359
tau=10^-5:10:100; %%% time array

for test=1:nrTest
    R1=[];
    R2=[];
    R3=[];
    realValue=[];
    clear test_laplace_fun
    if test==1
        F='1./s.^2';  %f(t) =t
        realValue=tau;
    elseif test==2
        F='1./((s+1).*(s+2))'; %f(t)=exp(-t)-exp(-2*t)
        realValue = exp(-tau) - exp(-2*tau);
    elseif test==3
        F='(s+1)./((s+2).^2+3^2)';%f(t)=exp(-t)*(cos(3*t)+(1-2)/3*sin(3*t))
        realValue =exp(-2*tau).*(cos(3*tau)+(1-2)/3*sin(3*tau));
        
    %elseif test==4
    %    F='(1./(s.*(s+1)))*((1/2*2*s.)-exp(-2*2*s)./(1-exp(-2*2*s)))'; 
    %    %f(t)= 1/(2*1) + (exp(-2*t)/2)*(1/(2*1*2) - exp(2*1*2)/(exp(2*1*2-1))) 
    %    %-2/pi*sum(sin(n*pi*t)/2-tan^-1(n*pi/2))/(n*sqrt(n^2*pi^2 +h^2))
    %    realValue = 1/(2*1) + (exp(-2*tau)/2)*(1/(2*1*2) - exp(2*1*2)/(exp(2*1*2-1)))
        
    end
    
    fid=fopen('test_laplace_fun.m','w');
    fprintf(fid,'function F=test_laplace_fun(s)');
    fprintf(fid,'\n');
    fprintf(fid,'	F=');
    fprintf(fid,F);
    fprintf(fid,';');
    fclose(fid);
    
    
    %%% Weeks's method, ADD REFERENCE
    tic
    N=100;
    [so bo]=wpar2(F,100,N,0,30,30,50);
    [R1 Est]=weekse(F,tau,N,so,bo);
    R1=R1';
    toc
    
    %%% De hoog's method, de Hoog, F. R., Knight, J. H., and Stokes, A. N. (1982). An improved
    %   method for numerical inversion of Laplace transforms. S.I.A.M. J. Sci.
    %   and Stat. Comput., 3, 357-366.
    tic
    R2=invlap('test_laplace_fun',tau);
    R2=R2';
    toc
    
    
    
    %%% Gaver-Stehfest's method, Stehfest, H., 1970, Algorithm 368: Numerical inversion of Laplace transform,
    %    Communication of the ACM, vol. 13 no. 1 p. 47-49
    tic
    L=20;
    for tidx=1:length(tau)
        R3(tidx) = gavsteh('test_laplace_fun',tau(tidx),L);
    end 
    toc
    
    %save results
    fileName = strcat('invLapTestResult_test',num2str(test),'.mat');
    save(fileName,'realValue', 'R1', 'R2', 'R3');
end