%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the autocovariance function using 2 methods:
% 1. Numerical Inverse Laplace Transform
% 2. Simulation
% Compare the two results. If they match, the code is good.
% copyright: hung.nguyen@adelaide.edu.au
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all

ONP=1/1; %%%% 1/ average length of the Lossy period
OFFP =1/100; %%%1/average length of the Lossless period
lambda = 0.1;
n_samples=1000;

%%%gamma dist parameters
ko=[5 1];
thetao=[1/(ko(1)*ONP)  1/(ko(2)*OFFP)];

k=ko;
theta=thetao;



%%%%%%% Get the autocovariance function
%%%% Using inverse Laplace Tranform
F=strcat(num2str(k(1)),'*',num2str(theta(1)),'./((',num2str(k(1)*theta(1) + k(2)*theta(2)),')*s)');
F=strcat(F,'-((',num2str(theta(1)),'*s+1).^',num2str(k(1)),'-1).*((',num2str(theta(2)),'*s+1).^',num2str(k(2)),'-1)');
F=strcat(F,'./','(s.^2*', '(',num2str(k(1)*theta(1) + k(2)*theta(2)),').*','((',num2str(theta(1)),'*s+1).^',num2str(k(1)),'.*(',num2str(theta(2)),'*s+1).^',num2str(k(2)),'-1))');


fid=fopen('temp_laplace_fun.m','w');
fprintf(fid,'function F=temp_laplace_fun(s)');
fprintf(fid,'\n');
fprintf(fid,'	F=');
fprintf(fid,F);
fprintf(fid,';');
fclose(fid);


%%% Get the variance
% Generate the samples
A_seq = cumsum(- log(rand(n_samples,1))/lambda); %% arrival time of samples;
ON_seq= mygamrnd(ko(1),thetao(1),2*n_samples,0);
OFF_seq= mygamrnd(ko(2),thetao(2),2*n_samples,0);

R_seq=[0];

idx=1;
for i=1:2*n_samples
    R_seq(idx+1) =R_seq(idx) + ON_seq(i);
    R_seq(idx+2)= R_seq(idx+1) + OFF_seq(i);
    idx=idx+2;
end

current_j=1;
for i=1:n_samples
    for j=current_j:length(R_seq)-1
        if ((A_seq(i) > R_seq(j))&&(A_seq(i)<=R_seq(j+1)))
            current_j=j;
            break;
        end
    end
    
    if (mod(current_j,2) == 1)
        samples(i)=1;
    else
        samples(i)=0;
    end
    
end

%Compute the max time lag
tauInt=1;
tauMax=floor(abs(trceTime(length(trceTime))-trceTime(1))/tauInt)+1;
tau=[10^-10:tauInt:tauMax];


%Inverse Laplace
tic
tau=[10^-10:tauInt:tauMax];
RTau1=invlap('temp_laplace_fun',tau)-(k(1)*theta(1)/sum(k.*theta))^2;
toc

%Simulation
tic
RTau2=GETAUTOCOV(k,theta,tau);
toc



%%%% Compute the variance
V_EST=0;
V_SIM=0;
for i=1:n_samples
    for j =1:n_samples
        idx=floor(abs(A_seq(i)-A_seq(j))/tauInt)+1;
        if idx<= length(RTau1)
            V_EST=V_EST+RTau1(idx);
            V_SIM=V_SIM+RTau2(idx);
        end
    end
end
VARINFER=V_EST/n_samples^2
VARSIM=V_SIM/n_samples^2



